@@ -7,13 +7,17 @@ from transformers import (
 from datasets import Dataset
 from torch.nn import functional as F
 
+
 models = {
     "es": {
         "sentiment": {
-            "model_name": "finiteautomata/beto-sentiment-analysis",
+            "model_name": "pysentimiento/robertuito-sentiment-analysis",
         },
         "emotion": {
-            "model_name": "finiteautomata/beto-emotion-analysis",
+            "model_name": "pysentimiento/robertuito-emotion-analysis",
+        },
+        "hate_speech": {
+            "model_name": "pysentimiento/robertuito-hate-speech",
         }
     },
     "en": {
@@ -29,39 +33,43 @@ models = {
     },
 }
 
-class BaseOutput:
+class AnalyzerOutput:
     """
     Base class for classification output
     """
-    def __init__(self, sentence, probas):
+    def __init__(self, sentence, probas, is_multilabel=False):
         """
         Constructor
         """
         self.sentence = sentence
         self.probas = probas
-        self.output = max(probas.items(), key=lambda x: x[1])[0]
+        self.is_multilabel = is_multilabel
+        if not is_multilabel:
+            self.output = max(probas.items(), key=lambda x: x[1])[0]
+        else:
+            self.output = [
+                k for k, v in probas.items() if v > 0.5
+            ]
+
 
     def __repr__(self):
         ret = f"{self.__class__.__name__}"
-        formatted_probas = sorted(self.probas.items(), key=lambda x: -x[1])
+        if not self.is_multilabel:
+            formatted_probas = sorted(self.probas.items(), key=lambda x: -x[1])
+        else:
+            formatted_probas = list(self.probas.items())
         formatted_probas = [f"{k}: {v:.3f}" for k, v in formatted_probas]
         formatted_probas = "{" + ", ".join(formatted_probas) + "}"
         ret += f"(output={self.output}, probas={formatted_probas})"
 
         return ret
 
-class SentimentOutput(BaseOutput):
-    pass
-
-class EmotionOutput(BaseOutput):
-    pass
-
 
 class Analyzer:
     """
     Wrapper to use sentiment analysis models as black-box
     """
-    def __init__(self, model_name, preprocessing_args={}, batch_size=32):
+    def __init__(self, model_name, task, preprocessing_args={}, batch_size=32):
         """
         Constructor for SentimentAnalyzer class
 
@@ -73,9 +81,13 @@ class Analyzer:
         self.tokenizer = AutoTokenizer.from_pretrained(model_name)
         self.tokenizer.model_max_length=128
         self.model = AutoModelForSequenceClassification.from_pretrained(model_name)
+
+        self.problem_type = self.model.config.problem_type
+
         self.id2label = self.model.config.id2label
         self.preprocessing_args = preprocessing_args
         self.batch_size = batch_size
+        self.task = task
 
         self.eval_trainer = Trainer(
             model=self.model,
@@ -86,11 +98,30 @@ class Analyzer:
             data_collator=DataCollatorWithPadding(self.tokenizer, padding="longest"),
         )
 
+
+
     def _tokenize(self, batch):
         return self.tokenizer(
             batch["text"], padding=False, truncation=True
         )
 
+    def _get_output(self, sentence, logits):
+        """
+        Get output from logits
+
+        It takes care of the type of problem: single or multi label classification
+        """
+        if self.problem_type == "multi_label_classification":
+            is_multilabel = True
+            probs = torch.sigmoid(logits).view(-1)
+        else:
+            is_multilabel = False
+            probs = torch.softmax(logits, dim=1).view(-1)
+
+        probas = {self.id2label[i]:probs[i].item() for i in self.id2label}
+        return AnalyzerOutput(sentence, probas=probas, is_multilabel=is_multilabel)
+
+
     def _predict_single(self, sentence):
         """
         Predict single
@@ -106,10 +137,8 @@ class Analyzer:
             )
         ).view(1, -1).to(device)
         output = self.model(idx)
-        probs = F.softmax(output.logits, dim=1).view(-1)
-        probas = {self.id2label[i]:probs[i].item() for i in self.id2label}
-
-        return self.output_class(sentence, probas=probas)
+        logits = output.logits
+        return self._get_output(sentence, logits)
 
 
     def predict(self, inputs):
@@ -135,85 +164,45 @@ class Analyzer:
         dataset = Dataset.from_dict({"text": sentences})
         dataset = dataset.map(self._tokenize, batched=True, batch_size=self.batch_size)
 
-        print(len(dataset))
 
         output = self.eval_trainer.predict(dataset)
         logits = torch.tensor(output.predictions)
-
-        probs = F.softmax(logits, dim=1)
-
-        probas = [{
-            self.id2label[i]: probs[j][i].item()
-            for i in self.id2label
-        } for j in range(probs.shape[0])]
-
-
-        rets = [self.output_class(sent, prob) for sent, prob in zip(sentences, probas) ]
+        rets = [self._get_output(sent, logits_row.view(1, -1)) for sent, logits_row in zip(sentences, logits)]
 
         return rets
 
 
-
-class SentimentAnalyzer(Analyzer):
+def create_analyzer(task, lang, model_name=None, preprocessing_args={}):
     """
-    Dummy class for sentiment analyzer
+    Create analyzer for the given task
+
+    Arguments:
+    ----------
+    task: str
+        Task name (sentiment or emotion)
+    lang: str
+        Language code (es or en)
+    model_name: str
+        Model name or path
+    preprocessing_args: dict
+        Preprocessing arguments
+
+    Returns:
+    --------
+        SentimentAnalyzer or EmotionAnalyzer
     """
-    output_class = SentimentOutput
-    def __init__(self, lang, model_name=None, preprocessing_args={}):
-        """
-        Class used to apply Sentiment Analysis to a tweet.
+    if lang not in models:
+        raise ValueError(f"Language {lang} not supported -- only supports {models.keys()}")
 
-        Currently supports 'es' (Spanish) and 'en' (English)
+    if task not in models[lang]:
+        raise ValueError(f"Task {task} not supported for {lang} -- only supports {models[lang].keys()}")
 
-        Arguments
-        ---------
+    preprocessing_args = preprocessing_args or {}
 
-        lang: str
-            Language used for the classifier. Must be one of 'es', 'en'
+    if not model_name:
+        model_info = models[lang][task]
+        model_name = model_info["model_name"]
+        preprocessing_args.update(model_info.get("preprocessing_args", {}))
 
-        model_name: str (default None)
-            If given, overrides default model_name for the language. Must be a local path or a huggingface's hub model name
-
-        preprocessing_args: dict (default {})
-            arguments to `preprocessing` function
-        """
-        if lang not in models:
-            raise ValueError(f"{lang} must be in {list(models.keys())}")
-
-        preprocessing_args["lang"] = lang
-
-        if not model_name:
-            model_info = models[lang]["sentiment"]
-            model_name = model_info["model_name"]
-            preprocessing_args.update(model_info.get("preprocessing_args", {}))
-
-        super().__init__(model_name, preprocessing_args=preprocessing_args)
-
-class EmotionAnalyzer(Analyzer):
-    """
-    Dummy class for emotion analyzer
-    """
-    output_class = EmotionOutput
-    def __init__(self, lang, model_name=None):
-        """
-        Class used to apply Emotion Analysis to a tweet.
-
-        Currently supports 'es' (Spanish) and 'en' (English)
-
-        Arguments
-        ---------
-
-        lang: str
-            Language used for the classifier. Must be one of 'es', 'en'
-
-        model_name: str (default None)
-            If given, overrides default model_name for the language. Must be a local path or a huggingface's hub model name
-        """
-        if lang not in models:
-            raise ValueError(f"{lang} must be in {list(models.keys())}")
-        if not model_name:
-            model_info = models[lang]["emotion"]
-            model_name = model_info["model_name"]
-            preprocessing_args = model_info.get("preprocessing_args", {})
-
-        super().__init__(model_name, preprocessing_args=preprocessing_args)
+    preprocessing_args["lang"] = lang
+    return Analyzer(model_name, task, preprocessing_args)
